#!/usr/bin/perl
#
#  Script for displaying the description of a package.
#
#  This file is part of the SuSE help system.
#
#  Copyright (C) 2002  SuSE Linux AG, Nuernberg
#
#  Author: Cornelius Schumacher <cschum@suse.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use FindBin;
use lib $FindBin::Bin;

use substitution;
use susehelp;

use constant DESCRPATH => "/var/adm/current_package_descr/suse/setup/descr/";
use constant HELPALIAS => "/hilfe/";
use constant HELPPATH  => "/usr/share/doc/susehilf/";
use constant TEMPLPATH => "/usr/share/doc/susehilf/lib/";

sub getNext(*$);
sub getLongNext(*$$);

my $helpurl="file:/" . HELPPATH;

&checkQuery();
&parseQuery();

my $language = lc getQuery( "lang", "en" );
my $package = getQuery( "package" );

if ( !$package ) {
  printErrorPage( "Need package argument." );
}

# if called from Web-Server we use aliases for url's
$helpurl = HELPALIAS if ($ENV{SERVER_PROTOCOL});

# now check for translation files
setLanguage( $language, "pac.po" );

unless (open(PKDLOCALE, DESCRPATH . $language . ".pkd")) {
  open(PKDLOCALE, DESCRPATH . "en.pkd") or die "could not even open en.pkd (as fallback)";
}

open(PKDCOMMON, DESCRPATH . "common.pkd") or die "could not open common.pkd";

# find package and all needed information
my $found = 0;
while (<PKDLOCALE>) {
  if (/Filename:\s*$package/i) {
    $found++;
    last;
  }
}
while (<PKDCOMMON>) {
  if (/RpmName:\s*$package/i) {
    $found++;
    last;
  }
}


# now render a html page
readSymbols( $susehelp::susehelpsymbols );

my $title = susehelp::i18n('Package: ') . $package;
my $imgdir = $helpurl . "img";
#my $charset = susehelp::i18n('iso-8859-1');

my $out = "--HEADER:--\n";

$out .= "--PACSHOW_TOP:--\n";

if ($found == 2) {
  my $authors = getNext(PKDCOMMON, "AuthorName:");
  my $size = getNext(PKDCOMMON, "Size:");
  my $copyright = getNext(PKDCOMMON, "Copyright:");
  my $version = getNext(PKDCOMMON, "Version:");
  my $series = getNext(PKDCOMMON, "Series:");
  my $label = getNext(PKDLOCALE, "Label:");
  my $notify = getLongNext(PKDLOCALE, "Notify:", "Yfiton:");
  my $delnotify = getLongNext(PKDLOCALE, "Delnotify:", "Yfitonled:");
  my $description = getLongNext(PKDLOCALE, "Description:", "Noitpircsed:");
  $title .=  " ($series)";

  # delete the "contains part" of description
  my $temptrans = susehelp::i18n('Contents: ');
  $description =~ s/(.*)$temptrans.*/$1/;

  registerSymbol( "PACSHOW_TITLE", $title );
  registerSymbol( "PACSHOW_HELLO", $label );

  $out .= "$description\n<hr>\n";
  $out .= "<b>" . susehelp::i18n('Author: ') . "</b> $authors\n<br>\n" if ($authors);
  $out .= "<b>Copyright: </b>$copyright\n<br>\n" if ($copyright);
  $out .= "<b>" . susehelp::i18n('Version: ') . "</b>$version\n<br>\n" if ($version);

  if ($notify) {
    $out .= "<hr>\n" . susehelp::i18n('Please note this when you <b>install</b> the package:') . "<br>\n<p>\n$notify\n";
  }
  
  if ($delnotify) {
    $out .= "<hr>\n" . susehelp::i18n('Please note this when you <b>uninstall</b> the package: ') . "<br>\n<p>\n$delnotify\n";
  }

  if ($size) {
    # format size
    $size =~ s/(\d*).*/$1/;
    $size = int ($size / 1024);
    $out .= "<hr>\n<b>" . susehelp::i18n('Required discspace: ') . "</b> $size KB<br>\n";
  }

  #
  # now check if package is installed and if it is get a long list of all files contained

  qx|rpm -q $package 2>/dev/null|;
  if ($?) {
    $out .= "<hr>\n<b>" . susehelp::i18n('Package is not installed') . "</b>\n";
  } else {
    my @filelist = qx|ls -ld `rpm -ql $package`|;
    # well we maybe do not have the right to show all files
    # die "could not execute rpm -ql" if $?;
    $out .= "<hr>\n<b>" . susehelp::i18n('Contents: ') . "</b>\n<pre>\n";
    $out .= join('', @filelist);
    $out .= "</pre>\n";
  }
} else {
  registerSymbol( "PACSHOW_TITLE", $title );
  registerSymbol( "PACSHOW_HELLO", susehelp::i18n('Package not found') );
}

$out .= "--PACSHOW_BOTTOM:--";
$out .= "--FOOTER:--";

&printCgiHeader;

print substituteSymbols( $out );

close(PKDCOMMON) or die "could not close common.pkd";
close(PKDLOCALE) or die "could not close " . $language . ".pkd";


#
# getNext
# return value of next entry with the given name
sub getNext(*$) {
  my $filehandle = shift;
  my $word = shift;

  my $pos = tell $filehandle;

  while (<$filehandle>) {
    if ($_ =~ /$word\s*(.*?)\s*$/i) {
      seek($filehandle, $pos, 0);
      return ($1) ? cleanText($1) : "";
    }
  }
  die "could not find $word";
}

#
# getLongNext
# get next multiline entry
sub getLongNext(*$$) {
  my $filehandle = shift;
  my $startword = shift;
  my $endword = shift;

  my $pos = tell $filehandle;

  my $result = "";
  my $doread = 0;

  my $line;
  
  while (1) {
    my $line = <$filehandle>;

    if ( !$line || $line =~ /^##/ ) { last; }

    if ( $line =~ /$startword/ ) {
      $doread = 1;
      next;
    }
    if ( $line =~ /$endword/ ) {
      last;
    }
    if ( $doread ) {
      $result .= $line;
    }    
  } 
  seek($filehandle, $pos, 0);

  return cleanText($result);
}

#
# cleanText
# changes some problems with special characters in HTML
sub cleanText($) {
  my $text = shift;
  $text =~ s/</&lt;/g if ($nohtml==0);
  $text =~ s/>/&gt;/g if ($nohtml==0);
  $text =~ s/\\n/<BR>/g if ($nohtml==0);
  return $text;
}

